#ifndef CUSTOMGRAPHICSITEMDEMO_H
#define CUSTOMGRAPHICSITEMDEMO_H

#include <QWidget>
#include <QGraphicsScene>

#include <QGraphicsItem>

namespace Ui {
class CustomGraphicsItemDemo;
}

class AreaSelector : public QObject, public QGraphicsItem {
    Q_OBJECT

    static const int RESIZE_ZONE_SIZE;
    static const int MIN_AREA_SIZE;

public:
    explicit AreaSelector( const QRectF& initialRect );
    ~AreaSelector();

    QRectF boundingRect() const;

    QRectF getSelectedArea() const {
        return m_innerRect;
    }

signals:
    void selectedAreaChanged( const QRectF& area );

protected:
    void paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget = 0 );

    void hoverEnterEvent( QGraphicsSceneHoverEvent* event );
    void hoverLeaveEvent( QGraphicsSceneHoverEvent* event );

    void mousePressEvent( QGraphicsSceneMouseEvent* event );
    void mouseMoveEvent( QGraphicsSceneMouseEvent* event );
    void mouseReleaseEvent( QGraphicsSceneMouseEvent* event );

private:
    enum SelectorZone {
        NONE,
        MOVE_ZONE,
        RESIZE_TOP_LEFT_ZONE,
        RESIZE_TOP_RIGHT_ZONE,
        RESIZE_BOTTOM_LEFT_ZONE,
        RESIZE_BOTTOM_RIGHT_ZONE,
    };

    QHash< SelectorZone, QRectF > generateZones() const;

private:
    QRectF m_innerRect;
    bool m_hovered;

    SelectorZone m_activeZone;
    QPointF m_centerOffset;

};

// ********************************************************************************

class CustomGraphicsItemDemo : public QWidget {
    Q_OBJECT

public:
    explicit CustomGraphicsItemDemo( QWidget* parent = 0 );
    ~CustomGraphicsItemDemo();

protected:
    void resizeEvent( QResizeEvent* );

private slots:
    void onSelectedAreaChanged( const QRectF& area );

private:
    Ui::CustomGraphicsItemDemo* ui;

    QGraphicsScene m_scene;
    QPixmap m_pix;
};

#endif // CUSTOMGRAPHICSITEMDEMO_H
